@@ -1,8 +1,9 @@
 public func CanBeOwned(){return true;}
 
-public func OnOwnerChanged(int old_owner)
+public func OnOwnerChanged(int new_owner, int old_owner)
 {
 	// ...
+	return _inherited(new_owner, old_owner, ...);
 }
 
 public func IsInteractable(object clonk)
@@ -15,4 +16,5 @@ func Initialize()
 {
 	// set right owner
 	SetOwner(GetOwnerOfPosition(0, 0));
+	return _inherited(...);
 }
\ No newline at end of file
