@@ -371,6 +371,7 @@ private func Produce(id product)
 	CheckComponents(product, true);
 	CheckFuel(product, true);
 	CheckLiquids(product, true);
+	CheckMaterials(product, true);
 	
 	// Add production effect.
 	AddEffect("ProcessProduction", this, 100, 2, this, nil, product);
@@ -384,8 +385,7 @@ private func CheckComponents(id product, bool remove)
 	{
 		var mat_id = item[0];
 		var mat_cost = item[1];
-		var mat_av = ObjectCount(Find_Container(this), Find_ID(mat_id));
-		if (mat_av < mat_cost)
+		if (!CheckComponent(mat_id, mat_cost))
 			return false; // Components missing.
 		else if (remove)
 		{
@@ -396,7 +396,13 @@ private func CheckComponents(id product, bool remove)
 	return true;
 }
 
-private func CheckFuel(id product, bool remove)
+public func CheckComponent(id component, int amount)
+{
+	// check if at least the given amount of the given component is available to be used for production
+	return (ObjectCount(Find_Container(this), Find_ID(component)) >= amount);
+}
+
+public func CheckFuel(id product, bool remove)
 {
 	if (FuelNeed(product) > 0)
 	{
@@ -422,7 +428,7 @@ private func CheckFuel(id product, bool remove)
 	return true;
 }
 
-private func CheckLiquids(id product, bool remove)
+public func CheckLiquids(id product, bool remove)
 {
 	var liq_need = LiquidNeed(product);
 	if (liq_need)
@@ -434,6 +440,10 @@ private func CheckLiquids(id product, bool remove)
 		for (var liq_container in FindObjects(Find_Container(this), Find_Func("IsLiquidContainer")))
 			if (liq_container->~GetBarrelMaterial() == liquid)
 				liquid_amount += liq_container->~GetFillLevel();
+		// Find objects that "are" liquid (e.g. ice)
+		for (var liq_object in FindObjects(Find_Container(this), Find_Func("IsLiquid")))
+			if (liq_object->~IsLiquid() == liquid)
+				liquid_amount += liq_object->~GetLiquidAmount();
 		if (liquid_amount < need)
 			return false;
 		else if (remove)
@@ -445,14 +455,22 @@ private func CheckLiquids(id product, bool remove)
 				var val = liq_container->~GetLiquid(liquid, need - extracted);
 				extracted += val[1];
 				if (extracted >= need)
-					break;			
-			}			
+					return true;
+			}
+			for (var liq_object in FindObjects(Find_Container(this), Find_Func("IsLiquid")))
+			{
+				if (liq_object->~IsLiquid() != liquid) continue;
+				extracted += liq_object->~GetLiquidAmount();
+				liq_object->RemoveObject();
+				if (extracted >= need)
+					break;
+			}
 		}		
 	}
 	return true;
 }
 
-private func CheckMaterials(id product, bool remove)
+public func CheckMaterials(id product, bool remove)
 {
 	var mat_need = MaterialNeed(product);
 	if (mat_need)
@@ -473,7 +491,7 @@ private func CheckMaterials(id product, bool remove)
 			for (var mat_container in FindObjects(Find_Container(this), Find_Func("IsMaterialContainer")))
 			{
 				var val = mat_container->~RemoveContainedMaterial(material, need - extracted);
-				extracted += val[1];
+				extracted += val;
 				if (extracted >= need)
 					break;
 			}
@@ -589,7 +607,12 @@ public func OnProductEjection(object product)
 	{
 		var x = GetX();
 		var y = GetY() + GetDefHeight()/2 - product->GetDefHeight()/2;
-		product->SetPosition(x, y);	
+		product->SetPosition(x, y);
+		// Sometimes, there is material in front of the building. Move vehicles upwards in that case
+		var max_unstick_range = Max(GetDefHeight()/5,5); // 8 pixels for tools workshop
+		var y_off = 0;
+		while (product->Stuck() && y_off < max_unstick_range)
+			product->SetPosition(x, y-++y_off);
 	}
 	// Items should stay inside.
 	else
@@ -671,6 +694,14 @@ protected func RejectEntrance(object obj)
 			if (FuelNeed(product) > 0)
 				return false;
 	}
+	// Liquid objects may be collected if a product needs them.
+	if (obj->~IsLiquid())
+	{
+		for (var product in GetProducts())
+			if (LiquidNeed(product))
+				if (LiquidNeed(product)[0] == obj->~IsLiquid())
+					return false;
+	}
 	// Liquid containers may be collected if a product needs them.
 	if (obj->~IsLiquidContainer())
 	{
