@@ -2,10 +2,19 @@
 	Structure Library
 	Basic library for structures, handles:
 	* Damage
+	* Energy bar if rule active
 	
 	@author Maikel
 */
 
+func Initialize()
+{
+	if (FindObject(Find_ID(Rule_StructureHPBars)))
+		if (this.HitPoints != nil)
+			AddEnergyBar();
+	return _inherited(...);
+}
+
 public func Damage(int change, int cause, int cause_plr)
 {
 	// Only do stuff if the object has the HitPoints property.
@@ -16,4 +25,7 @@ public func Damage(int change, int cause, int cause_plr)
 			return RemoveObject();
 		}
 	return _inherited(change, cause, cause_plr);
-}
\ No newline at end of file
+}
+
+// This object is a structure.
+public func IsStructure() { return true; }
\ No newline at end of file
