@@ -1,26 +1,18 @@
 /*
  * OpenClonk, http://www.openclonk.org
  *
- * Copyright (c) 1998-2000, 2003-2005, 2007  Matthes Bender
- * Copyright (c) 2001-2003, 2005-2008  Sven Eberhardt
- * Copyright (c) 2002-2008  Peter Wortmann
- * Copyright (c) 2004-2009, 2011  Günther Brammer
- * Copyright (c) 2005  Armin Burgmeier
- * Copyright (c) 2010  Benjamin Herr
- * Copyright (c) 2010  Carl-Philip Hänsch
- * Copyright (c) 2010-2011  Nicolas Hake
- * Copyright (c) 2001-2009, RedWolf Design GmbH, http://www.clonk.de
+ * Copyright (c) 1998-2000, Matthes Bender
+ * Copyright (c) 2001-2009, RedWolf Design GmbH, http://www.clonk.de/
+ * Copyright (c) 2009-2013, The OpenClonk Team and contributors
  *
- * Portions might be copyrighted by other authors who have contributed
- * to OpenClonk.
+ * Distributed under the terms of the ISC license; see accompanying file
+ * "COPYING" for details.
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- * See isc_license.txt for full license and disclaimer.
+ * "Clonk" is a registered trademark of Matthes Bender, used with permission.
+ * See accompanying file "TRADEMARK" for details.
  *
- * "Clonk" is a registered trademark of Matthes Bender.
- * See clonk_trademark_license.txt for full license.
+ * To redistribute this file separately, substitute the full license texts
+ * for the above references.
  */
 
 /* Handles group files */
@@ -567,7 +559,7 @@ bool C4Group::Open(const char *szGroupName, bool fCreate)
 	char szRealGroup[_MAX_FNAME];
 	SCopy(szGroupNameN,szRealGroup,_MAX_FNAME);
 	do
-		{ if (!TruncatePath(szRealGroup)) return Error("Open: File not found"); }
+		{ if (!TruncatePath(szRealGroup)) return Error(FormatString("Open(\"%s\"): File not found", szGroupNameN).getData()); }
 	while (!FileExists(szRealGroup));
 
 	// Open mother and child in exclusive mode
@@ -576,9 +568,9 @@ bool C4Group::Open(const char *szGroupName, bool fCreate)
 		return Error("Open: mem");
 	pMother->SetStdOutput(StdOutput);
 	if (!pMother->Open(szRealGroup))
-		{ Clear(); return Error("Open: Cannot open mother"); }
+		{ Clear(); Error(pMother->ErrorString); delete pMother; return false; }
 	if (!OpenAsChild(pMother,szGroupNameN+SLen(szRealGroup)+1,true))
-		{ Clear(); return Error("Open:: Cannot open as child"); }
+		{ Clear(); return false; }
 
 	// Success
 	return true;
@@ -1589,7 +1581,7 @@ bool C4Group::ExtractEntry(const char *szFilename, const char *szExtractTo)
 	case GRPF_File: // Copy entry to target
 		// Get entry
 		C4GroupEntry *pEntry;
-		if (!(pEntry=GetEntry(szFilename))) return false;
+		if (!(pEntry=GetEntry(szFilename))) return Error("Extract: Entry not found");
 		// Create dummy file to reserve target file name
 		hDummy.Create(szTargetFName,false);
 		hDummy.Write("Dummy",5);
@@ -2098,7 +2090,7 @@ bool C4Group::EnsureChildFilePtr(C4Group *pChild)
 	if ( !ItemIdentical( StdFile.Name, szChildPath ) )
 	{
 		// Reopen correct child stdfile
-		if ( !SetFilePtr2Entry( GetFilename(pChild->FileName) ) )
+		if ( !SetFilePtr2Entry( GetFilename(pChild->FileName), true ) )
 			return false;
 		// Advance to child's old file ptr
 		if ( !AdvanceFilePtr( pChild->EntryOffset + pChild->FilePtr ) )
@@ -2118,7 +2110,7 @@ StdStrBuf C4Group::GetFullName() const
 	{
 		if (*str) SInsert(str, sep, 0, _MAX_PATH);
 		// Avoid double slash
-		if (SLen(pGroup->FileName) > 1 || pGroup->FileName[0] != '/')
+		if (pGroup == this || SLen(pGroup->FileName) > 1 || pGroup->FileName[0] != '/')
 			SInsert(str, pGroup->FileName, 0, _MAX_PATH);
 		if (pGroup->Status == GRPF_Folder) break; // Folder is assumed to have full path
 	}
