@@ -1,25 +1,18 @@
 /*
  * OpenClonk, http://www.openclonk.org
  *
- * Copyright (c) 1998-2000, 2003  Matthes Bender
- * Copyright (c) 2001, 2005-2007  Sven Eberhardt
- * Copyright (c) 2004-2005, 2007  Peter Wortmann
- * Copyright (c) 2005-2011  Günther Brammer
- * Copyright (c) 2006, 2010  Armin Burgmeier
- * Copyright (c) 2009  Nicolas Hake
- * Copyright (c) 2010  Benjamin Herr
- * Copyright (c) 2001-2009, RedWolf Design GmbH, http://www.clonk.de
+ * Copyright (c) 1998-2000, Matthes Bender
+ * Copyright (c) 2001-2009, RedWolf Design GmbH, http://www.clonk.de/
+ * Copyright (c) 2009-2013, The OpenClonk Team and contributors
  *
- * Portions might be copyrighted by other authors who have contributed
- * to OpenClonk.
+ * Distributed under the terms of the ISC license; see accompanying file
+ * "COPYING" for details.
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- * See isc_license.txt for full license and disclaimer.
+ * "Clonk" is a registered trademark of Matthes Bender, used with permission.
+ * See accompanying file "TRADEMARK" for details.
  *
- * "Clonk" is a registered trademark of Matthes Bender.
- * See clonk_trademark_license.txt for full license.
+ * To redistribute this file separately, substitute the full license texts
+ * for the above references.
  */
 
 /* Handles viewport editing in console mode */
@@ -160,7 +153,7 @@ bool C4EditCursor::Move(float iX, float iY, DWORD dwKeyState)
 			Target = ((dwKeyState & MK_SHIFT) && Selection.Last) ? Selection.Last->Obj : NULL;
 			do
 			{
-				Target = Game.FindObject(C4ID::None,X,Y,0,0,OCF_NotContained, Target);
+				Target = Game.FindObject(NULL,X,Y,0,0,OCF_NotContained, Target);
 			}
 			while ((dwKeyState & MK_SHIFT) && Target && Selection.GetLink(Target));
 		}
@@ -213,6 +206,37 @@ void C4EditCursor::OnSelectionChanged()
 	Console.ObjectListDlg.Update(Selection);
 }
 
+void C4EditCursor::AddToSelection(C4Object *add_obj)
+{
+	if (!add_obj || !add_obj->Status) return;
+	// add object to selection and do script callback
+	Selection.Add(add_obj, C4ObjectList::stNone);
+	::Control.DoInput(CID_EMMoveObj, new C4ControlEMMoveObject(EMMO_Select, Fix0, Fix0, add_obj), CDT_Decide);
+}
+
+bool C4EditCursor::RemoveFromSelection(C4Object *remove_obj)
+{
+	if (!remove_obj || !remove_obj->Status) return false;
+	// remove object from selection and do script callback
+	if (!Selection.Remove(remove_obj)) return false;
+	::Control.DoInput(CID_EMMoveObj, new C4ControlEMMoveObject(EMMO_Deselect, Fix0, Fix0, remove_obj), CDT_Decide);
+	return true;
+}
+
+void C4EditCursor::ClearSelection()
+{
+	// remove all objects from selection and do script callbacks
+	// iterate safely because callback might delete selected objects!
+	C4Object *obj;
+	while ((obj = Selection.GetObject(0)))
+	{
+		Selection.Remove(obj);
+		if (obj->Status)
+			::Control.DoInput(CID_EMMoveObj, new C4ControlEMMoveObject(EMMO_Deselect, Fix0, Fix0, obj), CDT_Decide);
+	}
+	Selection.Clear();
+}
+
 bool C4EditCursor::LeftButtonDown(DWORD dwKeyState)
 {
 
@@ -227,8 +251,8 @@ bool C4EditCursor::LeftButtonDown(DWORD dwKeyState)
 		{
 			// Toggle target
 			if (Target)
-				if (!Selection.Remove(Target))
-					Selection.Add(Target, C4ObjectList::stNone);
+				if (!RemoveFromSelection(Target))
+					AddToSelection(Target);
 		}
 		else
 		{
@@ -241,11 +265,11 @@ bool C4EditCursor::LeftButtonDown(DWORD dwKeyState)
 						break;
 				}
 				if(!it) // means loop didn't break
-					{ Selection.Clear(); Selection.Add(Target, C4ObjectList::stNone); }
+					{ ClearSelection(); AddToSelection(Target); }
 			}
 			// Click on nothing: drag frame
 			if (!Target)
-				{ Selection.Clear(); DragFrame=true; X2=X; Y2=Y; }
+				{ ClearSelection(); DragFrame=true; X2=X; Y2=Y; }
 		}
 		break;
 		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -293,10 +317,10 @@ bool C4EditCursor::RightButtonDown(DWORD dwKeyState)
 				// Click on unselected
 				if (Target && !Selection.GetLink(Target))
 				{
-					Selection.Clear(); Selection.Add(Target, C4ObjectList::stNone);
+					ClearSelection(); AddToSelection(Target);
 				}
 				// Click on nothing
-				if (!Target) Selection.Clear();
+				if (!Target) ClearSelection();
 			}
 		}
 		break;
@@ -344,11 +368,9 @@ bool C4EditCursor::LeftButtonUp(DWORD dwKeyState)
 
 bool C4EditCursor::KeyDown(C4KeyCode KeyCode, DWORD dwKeyState)
 {
-	C4KeyCodeEx kcx(KeyCode);
-
 	// alt check
 	bool fAltIsDown = (dwKeyState & MK_ALT) != 0;
-	fAltIsDown = fAltIsDown || (kcx.ToString(false, false) == "Alt" || kcx.ToString(false, false) == "Alt_L" || kcx.ToString(false, false) == "Alt_R");
+	fAltIsDown = fAltIsDown || KeyCode == K_ALT_L || KeyCode == K_ALT_R;
 	if (fAltIsDown != fAltWasDown)
 	{
 		if ((fAltWasDown = fAltIsDown))
@@ -359,7 +381,7 @@ bool C4EditCursor::KeyDown(C4KeyCode KeyCode, DWORD dwKeyState)
 
 	// shift check
 	bool fShiftIsDown = (dwKeyState & MK_SHIFT) != 0;
-	fShiftIsDown = fShiftIsDown || (kcx.ToString(false, false) == "Shift" || kcx.ToString(false, false) == "Shift_L" || kcx.ToString(false, false) == "Shift_R");
+	fShiftIsDown = fShiftIsDown || KeyCode == K_SHIFT_L || KeyCode == K_SHIFT_R;
 	if(fShiftIsDown != fShiftWasDown)
 		fShiftWasDown = fShiftIsDown;
 
@@ -368,11 +390,9 @@ bool C4EditCursor::KeyDown(C4KeyCode KeyCode, DWORD dwKeyState)
 
 bool C4EditCursor::KeyUp(C4KeyCode KeyCode, DWORD dwKeyState)
 {
-	C4KeyCodeEx kcx(KeyCode);
-
 	// alt check
 	bool fAltIsDown = (dwKeyState & MK_ALT) != 0;
-	fAltIsDown = fAltIsDown && !(kcx.ToString(false, false) == "Alt" || kcx.ToString(false, false) == "Alt_L" || kcx.ToString(false, false) == "Alt_R");
+	fAltIsDown = fAltIsDown && !( KeyCode == K_ALT_L || KeyCode == K_ALT_R);
 	if (fAltIsDown != fAltWasDown)
 	{
 		if ((fAltWasDown = fAltIsDown))
@@ -383,7 +403,7 @@ bool C4EditCursor::KeyUp(C4KeyCode KeyCode, DWORD dwKeyState)
 
 	// shift check
 	bool fShiftIsDown = (dwKeyState & MK_SHIFT) != 0;
-	fShiftIsDown = fShiftIsDown && !(kcx.ToString(false, false) == "Shift" || kcx.ToString(false, false) == "Shift_L" || kcx.ToString(false, false) == "Shift_R");
+	fShiftIsDown = fShiftIsDown && !(KeyCode == K_SHIFT_L || KeyCode == K_SHIFT_R);
 	if(fShiftIsDown != fShiftWasDown)
 		fShiftWasDown = fShiftIsDown;
 
@@ -544,13 +564,13 @@ void C4EditCursor::MoveSelection(C4Real XOff, C4Real YOff)
 
 void C4EditCursor::FrameSelection()
 {
-	Selection.Clear();
+	ClearSelection();
 	C4Object *cobj; C4ObjectLink *clnk;
 	for (clnk=::Objects.First; clnk && (cobj=clnk->Obj); clnk=clnk->Next)
 		if (cobj->Status) if (cobj->OCF & OCF_NotContained)
 			{
 				if (Inside(cobj->GetX(),Min(X,X2),Max(X,X2)) && Inside(cobj->GetY(),Min(Y,Y2),Max(Y,Y2)))
-					Selection.Add(cobj, C4ObjectList::stNone);
+					AddToSelection(cobj);
 			}
 	OnSelectionChanged();
 }
