@@ -1,30 +1,18 @@
 /*
  * OpenClonk, http://www.openclonk.org
  *
- * Copyright (c) 2006-2007  Sven Eberhardt
- * Copyright (c) 2006, 2008-2010  Günther Brammer
- * Copyright (c) 2006-2007  Peter Wortmann
- * Copyright (c) 2006  Florian Groß
- * Copyright (c) 2007-2008  Matthes Bender
- * Copyright (c) 2010  Benjamin Herr
- * Copyright (c) 2010  Julius Michaelis
- * Copyright (c) 2010  Tobias Zwick
- * Copyright (c) 2010  Armin Burgmeier
- * Copyright (c) 2006-2009, RedWolf Design GmbH, http://www.clonk.de
-
-Permission to use, copy, modify, and/or distribute this software for any
-purpose with or without fee is hereby granted, provided that the above
-copyright notice and this permission notice appear in all copies.
-
-THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
-WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
-ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
-WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
-ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
-OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
-
-"Clonk" is a registered trademark of Matthes Bender. */
+ * Copyright (c) 2006-2009, RedWolf Design GmbH, http://www.clonk.de/
+ * Copyright (c) 2009-2013, The OpenClonk Team and contributors
+ *
+ * Distributed under the terms of the ISC license; see accompanying file
+ * "COPYING" for details.
+ *
+ * "Clonk" is a registered trademark of Matthes Bender, used with permission.
+ * See accompanying file "TRADEMARK" for details.
+ *
+ * To redistribute this file separately, substitute the full license texts
+ * for the above references.
+ */
 // Startup screen for non-parameterized engine start: Network game selection dialog
 
 #include <C4Include.h>
@@ -273,11 +261,6 @@ bool C4StartupNetListEntry::OnReference()
 	// special masterserver handling
 	if (eQueryType == NRQT_Masterserver)
 	{
-		// masterserver is official: So check version
-		StdStrBuf updURL;
-		StdStrBuf versInf;
-		if (pRefClient->GetUpdateURL(&updURL) && pRefClient->GetVersion(&versInf))
-			pNetDlg->CheckVersionUpdate(updURL.getData(),versInf.getData());
 		// masterserver: schedule next query
 		sInfoText[1].Format(LoadResStr("IDS_NET_INFOGAMES"), (int) iNewRefCount);
 		SetTimeout(TT_Masterserver);
@@ -290,8 +273,10 @@ bool C4StartupNetListEntry::OnReference()
 		return false;
 	}
 	else
+	{
 		// no ref found on custom adress: Schedule re-check
 		SetTimeout(TT_RefReqWait);
+	}
 	return true;
 }
 
@@ -592,7 +577,7 @@ C4Network2Reference *C4StartupNetListEntry::GrabReference()
 
 // ----------- C4StartupNetDlg ---------------------------------------------------------------------------------
 
-C4StartupNetDlg::C4StartupNetDlg() : C4StartupDlg(LoadResStr("IDS_DLG_NETSTART")), pChatTitleLabel(NULL), pMasterserverClient(NULL), fIsCollapsed(false), fUpdatingList(false), iGameDiscoverInterval(0), tLastRefresh(0)
+C4StartupNetDlg::C4StartupNetDlg() : C4StartupDlg(LoadResStr("IDS_DLG_NETSTART")), pChatTitleLabel(NULL), pMasterserverClient(NULL), fIsCollapsed(false), fUpdatingList(false), iGameDiscoverInterval(0), tLastRefresh(0), fUpdateCheckPending(false)
 {
 	// ctor
 	// key bindings
@@ -750,6 +735,8 @@ C4StartupNetDlg::~C4StartupNetDlg()
 {
 	// disable notifies
 	Application.InteractiveThread.ClearCallback(Ev_HTTP_Response, this);
+	Application.InteractiveThread.RemoveProc(&pUpdateClient);
+
 	DiscoverClient.Close();
 	Application.Remove(this);
 	if (pMasterserverClient) delete pMasterserverClient;
@@ -762,14 +749,20 @@ C4StartupNetDlg::~C4StartupNetDlg()
 void C4StartupNetDlg::DrawElement(C4TargetFacet &cgo)
 {
 	// draw background
+	typedef C4GUI::FullscreenDialog Base;
+	Base::DrawElement(cgo);
+#if 0
 	DrawBackground(cgo, C4Startup::Get()->Graphics.fctNetBG);
+#endif
 }
 
 void C4StartupNetDlg::OnShown()
 {
 	// callback when shown: Start searching for games
 	C4StartupDlg::OnShown();
+	CheckVersionUpdate();
 	UpdateList();
+	UpdateUpdateButton(); // in case update check was finished before callback registration
 	UpdateMasterserver();
 	OnSec1Timer();
 	tLastRefresh = time(0);
@@ -871,7 +864,7 @@ void C4StartupNetDlg::UpdateMasterserver()
 	else
 	{
 		pMasterserverClient = new C4StartupNetListEntry(pGameSelList, NULL, this);
-		StdStrBuf strVersion; strVersion.Format("%d.%d.%d.%d", C4XVER1, C4XVER2, C4XVER3, C4XVER4);
+		StdStrBuf strVersion; strVersion.Format("%d.%d.%d", C4XVER1, C4XVER2, C4XVER3);
 		StdStrBuf strQuery; strQuery.Format("%s?version=%s&platform=%s", Config.Network.GetLeagueServerAddress(), strVersion.getData(), C4_OS);
 		pMasterserverClient->SetRefQuery(strQuery.getData(), C4StartupNetListEntry::NRQT_Masterserver);
 	}
@@ -985,9 +978,28 @@ C4StartupNetDlg::DlgMode C4StartupNetDlg::GetDlgMode()
 
 void C4StartupNetDlg::OnThreadEvent(C4InteractiveEventType eEvent, void *pEventData)
 {
+	UpdateUpdateButton();
 	UpdateList(true);
 }
 
+void C4StartupNetDlg::UpdateUpdateButton()
+{
+	if (!fUpdateCheckPending) return;
+	if(!pUpdateClient.isSuccess() || pUpdateClient.isBusy()) return;
+
+	pUpdateClient.SetNotify(NULL);
+
+	StdCopyStrBuf versionInfo;
+
+	pUpdateClient.GetVersion(&versionInfo);
+	pUpdateClient.GetUpdateURL(&UpdateURL);
+
+#ifdef WITH_AUTOMATIC_UPDATE
+	btnUpdate->SetVisibility(C4UpdateDlg::IsValidUpdate(versionInfo.getData()));
+#endif
+	fUpdateCheckPending = false;
+}
+
 bool C4StartupNetDlg::DoOK()
 {
 	// OK in chat mode? Forward to chat control
@@ -1202,26 +1214,18 @@ void C4StartupNetDlg::OnReferenceEntryAdd(C4StartupNetListEntry *pEntry)
 		pEntry->UpdateCollapsed(true);
 }
 
-void C4StartupNetDlg::CheckVersionUpdate(const char *szUpdateURL, const char *szVersion)
+void C4StartupNetDlg::CheckVersionUpdate()
 {
 #ifdef WITH_AUTOMATIC_UPDATE
-	// Is a valid update
-	if (C4UpdateDlg::IsValidUpdate(szVersion))
+	StdStrBuf strVersion; strVersion.Format("%d.%d.%d", C4XVER1, C4XVER2, C4XVER3);
+	StdStrBuf strQuery; strQuery.Format("%s?version=%s&platform=%s&action=version", Config.Network.UpdateServerAddress, strVersion.getData(), C4_OS);
+
+	if (pUpdateClient.Init() && pUpdateClient.SetServer(strQuery.getData()) && pUpdateClient.QueryUpdateURL())
 	{
-		UpdateURL = szUpdateURL;
-		btnUpdate->SetVisibility(true);
-	}
-	// Otherwise: no update available
-	else
-	{
-		btnUpdate->SetVisibility(false);
-	}
-#else
-	if(szUpdateURL && *szUpdateURL)
-	{
-		// TODO: We could show an item notifying the user that an
-		// update is available externally.
+		pUpdateClient.SetNotify(&Application.InteractiveThread);
+		Application.InteractiveThread.AddProc(&pUpdateClient);
 	}
+	fUpdateCheckPending = true;
 #endif
 }
 
