@@ -1,23 +1,18 @@
 /*
  * OpenClonk, http://www.openclonk.org
  *
- * Copyright (c) 1998-2000, 2007  Matthes Bender
- * Copyright (c) 2001-2002, 2005-2007, 2011  Sven Eberhardt
- * Copyright (c) 2006-2007  Peter Wortmann
- * Copyright (c) 2006-2007, 2009  Günther Brammer
- * Copyright (c) 2010-2011  Nicolas Hake
- * Copyright (c) 2001-2009, RedWolf Design GmbH, http://www.clonk.de
+ * Copyright (c) 1998-2000, Matthes Bender
+ * Copyright (c) 2001-2009, RedWolf Design GmbH, http://www.clonk.de/
+ * Copyright (c) 2009-2013, The OpenClonk Team and contributors
  *
- * Portions might be copyrighted by other authors who have contributed
- * to OpenClonk.
+ * Distributed under the terms of the ISC license; see accompanying file
+ * "COPYING" for details.
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- * See isc_license.txt for full license and disclaimer.
+ * "Clonk" is a registered trademark of Matthes Bender, used with permission.
+ * See accompanying file "TRADEMARK" for details.
  *
- * "Clonk" is a registered trademark of Matthes Bender.
- * See clonk_trademark_license.txt for full license.
+ * To redistribute this file separately, substitute the full license texts
+ * for the above references.
  */
 
 /* Material definitions used by the landscape */
@@ -35,7 +30,6 @@
 #include <C4Landscape.h>
 #include <C4SoundSystem.h>
 #include <C4Effect.h>
-#include <C4Game.h>
 #include <C4Log.h>
 #include <C4Physics.h> // For GravAccel
 
@@ -66,18 +60,18 @@ void C4MaterialReaction::CompileFunc(StdCompiler *pComp)
 	StdStrBuf sReactionFuncName;
 	int32_t i=0; while (ReactionFuncMap[i].szRFName && (ReactionFuncMap[i].pFunc != pFunc)) ++i;
 	sReactionFuncName = ReactionFuncMap[i].szRFName;
-	pComp->Value(mkNamingAdapt(sReactionFuncName,   "Type",                     StdStrBuf()     ));
+	pComp->Value(mkNamingAdapt(mkParAdapt(sReactionFuncName, StdCompiler::RCT_IdtfAllowEmpty),   "Type",                     StdCopyStrBuf() ));
 	i=0; while (ReactionFuncMap[i].szRFName && !SEqual(ReactionFuncMap[i].szRFName, sReactionFuncName.getData())) ++i;
 	pFunc = ReactionFuncMap[i].pFunc;
 	// compile the rest
-	pComp->Value(mkNamingAdapt(TargetSpec,          "TargetSpec",               StdCopyStrBuf() ));
-	pComp->Value(mkNamingAdapt(ScriptFunc,          "ScriptFunc",               StdCopyStrBuf() ));
+	pComp->Value(mkNamingAdapt(mkParAdapt(TargetSpec, StdCompiler::RCT_All),          "TargetSpec",               StdCopyStrBuf() ));
+	pComp->Value(mkNamingAdapt(mkParAdapt(ScriptFunc, StdCompiler::RCT_IdtfAllowEmpty),          "ScriptFunc",               StdCopyStrBuf() ));
 	pComp->Value(mkNamingAdapt(iExecMask,           "ExecMask",                 ~0u             ));
 	pComp->Value(mkNamingAdapt(fReverse,            "Reverse",                  false           ));
 	pComp->Value(mkNamingAdapt(fInverseSpec,        "InverseSpec",              false           ));
 	pComp->Value(mkNamingAdapt(fInsertionCheck,     "CheckSlide",               true            ));
 	pComp->Value(mkNamingAdapt(iDepth,              "Depth",                    0               ));
-	pComp->Value(mkNamingAdapt(sConvertMat,         "ConvertMat",               StdCopyStrBuf() ));
+	pComp->Value(mkNamingAdapt(mkParAdapt(sConvertMat, StdCompiler::RCT_IdtfAllowEmpty),         "ConvertMat",               StdCopyStrBuf() ));
 	pComp->Value(mkNamingAdapt(iCorrosionRate,      "CorrosionRate",            100             ));
 }
 
@@ -131,10 +125,10 @@ bool C4MaterialShape::Load(C4Group &group, const char *filename)
 			{
 				min = max = *j;
 			}
-			if (j ->x<- overlap_left     ) overlap_left   =- j ->x;
-			if (j ->y<- overlap_top      ) overlap_top    =- j ->y;
-			if (j ->x> wdt+overlap_right ) overlap_right  =  j ->x- wdt;
-			if (j ->y> hgt+overlap_bottom) overlap_bottom =  j ->y- hgt;
+			if (j ->x<- overlap_left     ) overlap_left   = -j->x;
+			if (j ->y<- overlap_top      ) overlap_top    = -j->y;
+			if (j ->x> wdt+overlap_right ) overlap_right  =  j->x - wdt;
+			if (j ->y> hgt+overlap_bottom) overlap_bottom =  j->y - hgt;
 		}
 		center.x /= n; center.y /= n;
 		i->center = center; i->min = min; i->max = max;
@@ -271,6 +265,7 @@ void C4MaterialCore::Clear()
 	TempConvStrength = 0;
 	MinHeightCount = 0;
 	SplashRate=10;
+	KeepSinglePixels=false;
 }
 
 void C4MaterialCore::Default()
@@ -370,6 +365,7 @@ void C4MaterialCore::CompileFunc(StdCompiler *pComp)
 	                                                "BelowTempConvertTo",  ""));
 	pComp->Value(mkNamingAdapt(MinHeightCount,      "MinHeightCount",      0));
 	pComp->Value(mkNamingAdapt(SplashRate,          "SplashRate",          10));
+	pComp->Value(mkNamingAdapt(KeepSinglePixels,    "KeepSinglePixels",    false));
 	pComp->NameEnd();
 	// material reactions
 	pComp->Value(mkNamingAdapt(mkSTLContainerAdapt(CustomReactionList),
@@ -483,7 +479,7 @@ int32_t C4MaterialMap::Get(const char *szMaterial)
 }
 
 
-bool C4MaterialMap::CrossMapMaterials() // Called after load
+bool C4MaterialMap::CrossMapMaterials(const char* szEarthMaterial) // Called after load
 {
 	// Check material number
 	if (::MaterialMap.Num>C4MaxMaterial)
@@ -502,8 +498,8 @@ bool C4MaterialMap::CrossMapMaterials() // Called after load
 			// natural stuff: material conversion here?
 			if (pMatPXS && pMatPXS->sInMatConvert.getLength() && SEqualNoCase(pMatPXS->sInMatConvert.getData(), pMatLS ? pMatLS->Name : C4TLS_MatSky))
 				pReaction = &DefReactConvert;
-			// the rest is happening for same/higher densities only
-			else if ((MatDensity(iMatPXS) <= MatDensity(iMatLS)) && pMatPXS && pMatLS)
+			// non-sky reactions
+			else if (pMatPXS && pMatLS)
 			{
 				// incindiary vs extinguisher
 				if ((pMatPXS->Incindiary && pMatLS->Extinguisher) || (pMatPXS->Extinguisher && pMatLS->Incindiary))
@@ -514,8 +510,11 @@ bool C4MaterialMap::CrossMapMaterials() // Called after load
 				// corrosive vs corrode
 				else if (pMatPXS->Corrosive && pMatLS->Corrode)
 					pReaction = &DefReactCorrode;
-				// otherwise, when hitting same or higher density: Material insertion
-				else
+				// liquid hitting liquid or solid: Material insertion
+				else if (DensityLiquid(MatDensity(iMatPXS)) && DensitySemiSolid(MatDensity(iMatLS)))
+					pReaction = &DefReactInsert;
+				// solid hitting solid: Material insertion
+				else if (DensitySolid(MatDensity(iMatPXS)) && DensitySolid(MatDensity(iMatLS)))
 					pReaction = &DefReactInsert;
 			}
 			// assign the function; or NULL for no reaction
@@ -588,9 +587,12 @@ bool C4MaterialMap::CrossMapMaterials() // Called after load
 			{
 				// single material target
 				if (pReact->fInverseSpec)
-					for (int32_t cnt2=-1; cnt2<Num; cnt2++) if (cnt2!=tmat) SetMatReaction(cnt, cnt2, pReact);
+					for (int32_t cnt2=-1; cnt2<Num; cnt2++) {
+						if (cnt2!=tmat)
+							SetMatReaction(cnt, cnt2, pReact);
 						else
 							SetMatReaction(cnt, tmat, pReact);
+					}
 			}
 			else if (SEqualNoCase(pReact->TargetSpec.getData(), "All"))
 			{
@@ -675,10 +677,11 @@ bool C4MaterialMap::CrossMapMaterials() // Called after load
 			if (ppReactionMap[(cnt2+1)*(Num+1) + cnt+1])
 				printf("%s -> %s: %p\n", Map[cnt].Name, Map[cnt2].Name, ppReactionMap[(cnt2+1)*(Num+1) + cnt+1]->pFunc);
 #endif
+
 	// Get hardcoded system material indices
-	const C4TexMapEntry* earth_entry = ::TextureMap.GetEntry(::TextureMap.GetIndexMatTex(Game.C4S.Landscape.Material));
+	const C4TexMapEntry* earth_entry = ::TextureMap.GetEntry(::TextureMap.GetIndexMatTex(szEarthMaterial));
 	if(!earth_entry)
-		{ LogFatal(FormatString("Earth material \"%s\" not found!", Game.C4S.Landscape.Material).getData()); return false; }
+		{ LogFatal(FormatString("Earth material \"%s\" not found!", szEarthMaterial).getData()); return false; }
 
 	MVehic   = Get("Vehicle"); MCVehic = Mat2PixColDefault(MVehic);
 	MTunnel  = Get("Tunnel");
@@ -687,6 +690,7 @@ bool C4MaterialMap::CrossMapMaterials() // Called after load
 
 	if ((MVehic==MNone) || (MTunnel==MNone))
 		{ LogFatal(LoadResStr("IDS_PRC_NOSYSMATS")); return false; }
+
 	return true;
 }
 
@@ -804,11 +808,14 @@ bool mrfInsertCheck(int32_t &iX, int32_t &iY, C4Real &fXDir, C4Real &fYDir, int3
 
 	// Incindiary mats smoke on contact even before doing their slide
 	if (::MaterialMap.Map[iPxsMat].Incindiary)
-		if (!Random(25)) Smoke(iX, iY, 4+Random(3) );
+		if (!Random(25))
+		{
+			Smoke(iX, iY, 4 + Random(3));
+		}
 
 	// Move by mat path/slide
 	int32_t iSlideX = iX, iSlideY = iY;
-	if (::Landscape.FindMatSlide(iSlideX,iSlideY,Sign(GravAccel),::MaterialMap.Map[iPxsMat].Density,::MaterialMap.Map[iPxsMat].MaxSlide))
+	if (::Landscape.FindMatSlide(iSlideX,iSlideY,Sign(GravAccel),Min(::MaterialMap.Map[iPxsMat].Density, C4M_Solid),::MaterialMap.Map[iPxsMat].MaxSlide))
 	{
 		if (iPxsMat == iLsMat)
 			{ iX = iSlideX; iY = iSlideY; fXDir = 0; return false; }
@@ -916,16 +923,19 @@ bool C4MaterialMap::mrfCorrode(C4MaterialReaction *pReaction, int32_t &iX, int32
 	case meeMassMove: // MassMover-movement
 	{
 		// evaluate corrosion percentage
-		bool fDoCorrode;
+		bool fDoCorrode; int d100 = Random(100);
 		if (pReaction->fUserDefined)
-			fDoCorrode = (Random(100) < pReaction->iCorrosionRate);
+			fDoCorrode = (d100 < pReaction->iCorrosionRate);
 		else
-			fDoCorrode = (Random(100) < ::MaterialMap.Map[iPxsMat].Corrosive) && (Random(100) < ::MaterialMap.Map[iLsMat].Corrode);
+			fDoCorrode = (d100 < ::MaterialMap.Map[iPxsMat].Corrosive) && (d100 < ::MaterialMap.Map[iLsMat].Corrode);
 		if (fDoCorrode)
 		{
 			ClearBackPix(iLSPosX,iLSPosY);
 			//::Landscape.CheckInstabilityRange(iLSPosX,iLSPosY); - more correct, but makes acid too effective as well
-			if (!Random(5)) Smoke(iX,iY,3+Random(3));
+			if (!Random(5))
+			{
+				Smoke(iX, iY, 3 + Random(3));
+			}
 			if (!Random(20)) StartSoundEffectAt("Corrode", iX, iY);
 			return true;
 		}
@@ -940,21 +950,24 @@ bool C4MaterialMap::mrfCorrode(C4MaterialReaction *pReaction, int32_t &iX, int32
 				// either splash or slide prevented interaction
 				return false;
 		// evaluate corrosion percentage
-		bool fDoCorrode;
+		bool fDoCorrode; int d100 = Random(100);
 		if (pReaction->fUserDefined)
-			fDoCorrode = (Random(100) < pReaction->iCorrosionRate);
+			fDoCorrode = (d100 < pReaction->iCorrosionRate);
 		else
-			fDoCorrode = (Random(100) < ::MaterialMap.Map[iPxsMat].Corrosive) && (Random(100) < ::MaterialMap.Map[iLsMat].Corrode);
+			fDoCorrode = (d100 < ::MaterialMap.Map[iPxsMat].Corrosive) && (d100 < ::MaterialMap.Map[iLsMat].Corrode);
 		if (fDoCorrode)
 		{
 			ClearBackPix(iLSPosX,iLSPosY);
 			::Landscape.CheckInstabilityRange(iLSPosX,iLSPosY);
-			if (!Random(5)) Smoke(iX,iY,3+Random(3));
+			if (!Random(5))
+			{
+				Smoke(iX,iY,3+Random(3));
+			}
 			if (!Random(20)) StartSoundEffectAt("Corrode", iX, iY);
 			return true;
 		}
 		// Else: dead. Insert material here
-		::Landscape.InsertMaterial(iPxsMat,iX,iY);
+		::Landscape.InsertMaterial(iPxsMat,&iX,&iY);
 		return true;
 	}
 	}
@@ -981,7 +994,7 @@ bool C4MaterialMap::mrfIncinerate(C4MaterialReaction *pReaction, int32_t &iX, in
 		// evaluate inflammation (should always succeed)
 		if (::Landscape.Incinerate(iX, iY)) return true;
 		// Else: dead. Insert material here
-		::Landscape.InsertMaterial(iPxsMat,iX,iY);
+		::Landscape.InsertMaterial(iPxsMat,&iX,&iY);
 		return true;
 	}
 	// not handled
@@ -1004,7 +1017,7 @@ bool C4MaterialMap::mrfInsert(C4MaterialReaction *pReaction, int32_t &iX, int32_
 				// continue existing
 				return false;
 		// Else: dead. Insert material here
-		::Landscape.InsertMaterial(iPxsMat,iX,iY);
+		::Landscape.InsertMaterial(iPxsMat,&iX,&iY);
 		return true;
 	}
 
