@@ -1,24 +1,18 @@
 /*
  * OpenClonk, http://www.openclonk.org
  *
- * Copyright (c) 1998-2000, 2003-2004, 2007  Matthes Bender
- * Copyright (c) 2002, 2004, 2007-2008  Sven Eberhardt
- * Copyright (c) 2004-2005  Peter Wortmann
- * Copyright (c) 2005, 2007, 2009, 2011  Günther Brammer
- * Copyright (c) 2010  Benjamin Herr
- * Copyright (c) 2010-2011  Nicolas Hake
- * Copyright (c) 2001-2009, RedWolf Design GmbH, http://www.clonk.de
+ * Copyright (c) 1998-2000, Matthes Bender
+ * Copyright (c) 2001-2009, RedWolf Design GmbH, http://www.clonk.de/
+ * Copyright (c) 2009-2013, The OpenClonk Team and contributors
  *
- * Portions might be copyrighted by other authors who have contributed
- * to OpenClonk.
+ * Distributed under the terms of the ISC license; see accompanying file
+ * "COPYING" for details.
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- * See isc_license.txt for full license and disclaimer.
+ * "Clonk" is a registered trademark of Matthes Bender, used with permission.
+ * See accompanying file "TRADEMARK" for details.
  *
- * "Clonk" is a registered trademark of Matthes Bender.
- * See clonk_trademark_license.txt for full license.
+ * To redistribute this file separately, substitute the full license texts
+ * for the above references.
  */
 
 /* All kinds of valuable helpers */
@@ -26,8 +20,6 @@
 #include "C4Include.h"
 #include <StdSync.h>
 
-#include <sys/timeb.h>
-
 //------------------------------------- Basics ----------------------------------------
 
 int32_t Distance(int32_t iX1, int32_t iY1, int32_t iX2, int32_t iY2)
