@@ -1,31 +1,41 @@
 /*
  * OpenClonk, http://www.openclonk.org
  *
- * Copyright (c) 2009-2011  Armin Burgmeier
- * Copyright (c) 2009  Mark Haßelbusch
- * Copyright (c) 2010  Nicolas Hake
- * Copyright (c) 2001-2009, RedWolf Design GmbH, http://www.clonk.de
+ * Copyright (c) 2001-2009, RedWolf Design GmbH, http://www.clonk.de/
+ * Copyright (c) 2009-2013, The OpenClonk Team and contributors
  *
- * Portions might be copyrighted by other authors who have contributed
- * to OpenClonk.
+ * Distributed under the terms of the ISC license; see accompanying file
+ * "COPYING" for details.
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- * See isc_license.txt for full license and disclaimer.
+ * "Clonk" is a registered trademark of Matthes Bender, used with permission.
+ * See accompanying file "TRADEMARK" for details.
  *
- * "Clonk" is a registered trademark of Matthes Bender.
- * See clonk_trademark_license.txt for full license.
+ * To redistribute this file separately, substitute the full license texts
+ * for the above references.
  */
 
 #include "C4Include.h"
 #include <StdMesh.h>
 #include <algorithm>
 
-static int StdMeshFaceCmp(const StdMeshFace& face1, const StdMeshFace& face2);
+namespace
+{
+	struct StdMeshFaceOrderHelper
+	{
+		float z;
+		unsigned int i;
+	};
+}
+
+static int StdMeshFaceCmp(const StdMeshFaceOrderHelper& h1, const StdMeshFaceOrderHelper& h2)
+{
+	if(h1.z < h2.z) return -1;
+	else if(h1.z > h2.z) return +1;
+	return 0;
+}
 
 #define SORT_NAME StdMesh
-#define SORT_TYPE StdMeshFace
+#define SORT_TYPE StdMeshFaceOrderHelper
 #define SORT_CMP StdMeshFaceCmp
 #include "timsort/sort.h"
 
@@ -42,72 +52,91 @@ namespace
 		}
 	};
 
-	// Helper to sort faces for FaceOrdering
-	struct StdMeshInstanceFaceOrderingCmpPred
+	// Helper to sort submesh instances so that opaque ones appear before non-opaque ones,
+	// this is required if materials are changed with SetMeshMaterial.
+	struct StdMeshSubMeshInstanceVisibilityCmpPred
 	{
-		const StdMeshVertex* m_vertices;
-		StdSubMeshInstance::FaceOrdering m_face_ordering;
-		const StdMeshMatrix& m_global_trans;
-
-		StdMeshInstanceFaceOrderingCmpPred(const StdMeshInstance& mesh_inst, const StdSubMeshInstance& sub_inst,
-		                                   StdSubMeshInstance::FaceOrdering face_ordering, const StdMeshMatrix& global_trans):
-				m_face_ordering(face_ordering), m_global_trans(global_trans)
+		bool operator()(const StdSubMeshInstance* first, const StdSubMeshInstance* second)
 		{
-			if(sub_inst.GetNumVertices() > 0)
-				m_vertices = &sub_inst.GetVertices()[0];
-			else
-				m_vertices = &mesh_inst.GetSharedVertices()[0];
-		}
-
-		inline float get_z(const StdMeshVertex& vtx) const
-		{
-			// We need to evaluate the Z coordinate of the transformed vertex
-			// (for all three vertices of the two faces), something like
-			// float z11 = (m_global_trans*m_vertices[face1.Vertices[0]]).z;
-			// However we don't do the full matrix multiplication as we are
-			// only interested in the Z coordinate of the result, also we are
-			// not interested in the resulting normals.
-			return m_global_trans(2,0)*vtx.x + m_global_trans(2,1)*vtx.y + m_global_trans(2,2)*vtx.z + m_global_trans(2,3);
-		}
-
-		bool operator()(const StdMeshFace& face1, const StdMeshFace& face2) const
-		{
-			return compare(face1, face2) < 0;
-		}
-
-		int compare(const StdMeshFace& face1, const StdMeshFace& face2) const
-		{
-			// TODO: Need to apply attach matrix in case of attached meshes
-			switch (m_face_ordering)
-			{
-			case StdSubMeshInstance::FO_Fixed:
-				assert(false);
-				return 0;
-			case StdSubMeshInstance::FO_FarthestToNearest:
-			case StdSubMeshInstance::FO_NearestToFarthest:
-			{
-				float z11 = get_z(m_vertices[face1.Vertices[0]]);
-				float z12 = get_z(m_vertices[face1.Vertices[1]]);
-				float z13 = get_z(m_vertices[face1.Vertices[2]]);
-				float z21 = get_z(m_vertices[face2.Vertices[0]]);
-				float z22 = get_z(m_vertices[face2.Vertices[1]]);
-				float z23 = get_z(m_vertices[face2.Vertices[2]]);
-
-				float z1 = std::max(std::max(z11, z12), z13);
-				float z2 = std::max(std::max(z21, z22), z23);
-
-				if (m_face_ordering == StdSubMeshInstance::FO_FarthestToNearest)
-					return (z1 < z2 ? -1 : (z1 > z2 ? +1 : 0));
-				else
-					return (z2 < z1 ? -1 : (z2 > z1 ? +1 : 0));
-			}
-			default:
-				assert(false);
-				return 0;
-			}
+			return first->GetMaterial().IsOpaque() > second->GetMaterial().IsOpaque();
 		}
 	};
 
+	float StdMeshFaceOrderGetVertexZ(const StdMeshVertex& vtx, const StdMeshMatrix& trans)
+	{
+		// TODO: Need to apply attach matrix in case of attached meshes
+
+		// We need to evaluate the Z coordinate of the transformed vertex
+		// (for all three vertices of the two faces), something like
+		// float z11 = (trans*m_vertices[face1.Vertices[0]]).z;
+		// However we don't do the full matrix multiplication as we are
+		// only interested in the Z coordinate of the result, also we are
+		// not interested in the resulting normals.
+		return trans(2,0)*vtx.x + trans(2,1)*vtx.y + trans(2,2)*vtx.z + trans(2,3);
+	}
+
+	float StdMeshFaceOrderGetFaceZ(const StdMeshVertex* vertices, const StdMeshFace& face, const StdMeshMatrix& trans)
+	{
+		const float z1 = StdMeshFaceOrderGetVertexZ(vertices[face.Vertices[0]], trans);
+		const float z2 = StdMeshFaceOrderGetVertexZ(vertices[face.Vertices[1]], trans);
+		const float z3 = StdMeshFaceOrderGetVertexZ(vertices[face.Vertices[2]], trans);
+		return std::max(std::max(z1, z2), z3);
+	}
+
+	void SortFacesArray(const StdMeshVertex* vertices, std::vector<StdMeshFace>& faces, StdSubMeshInstance::FaceOrdering face_ordering, const StdMeshMatrix& trans)
+	{
+		if(faces.empty()) return;
+
+		std::vector<StdMeshFaceOrderHelper> helpers(faces.size());
+		for(unsigned int i = 0; i < faces.size(); ++i)
+		{
+			helpers[i].i = i;
+			helpers[i].z = StdMeshFaceOrderGetFaceZ(vertices, faces[i], trans);
+		}
+
+		// The reason to use timsort here instead of std::sort is for performance
+		// reasons. This is performance critical code, with this function being
+		// called at least once per frame for each semi-transparent object. I have
+		// measured a factor 7 difference between the two sorting algorithms on my
+		// system.
+
+		// We also pre-compute the Z values that we use for sorting, and sort the
+		// array of Z values, then use the resorted indices to sort the original
+		// faces array. The reason for this is twofold:
+		// 1. We don't need to compute the Z value every time the comparison function
+		//    is called. Even though the computation is not very expensive, we have
+		//    to do many comparisons, and small things add up. I have measured a
+		//    5-10% performance benefit.
+		// 2. More importantly, due to floating point rounding errors we cannot guarantee
+		//    that Z values computed in the sorting function always yield the exact same
+		//    number, and the same sorting result for the same faces. This can lead to
+		//    a crash, because the f(a1, a2) = -f(a2, a1) property for the sorting function
+		//    would no longer be met, resulting in undefined behaviour in the sort call.
+		//    See http://bugs.openclonk.org/view.php?id=984.
+		StdMesh_tim_sort(&helpers[0], helpers.size());
+
+		std::vector<StdMeshFace> new_faces(faces.size());
+		switch(face_ordering)
+		{
+		case StdSubMeshInstance::FO_Fixed:
+			assert(false);
+			break;
+		case StdSubMeshInstance::FO_FarthestToNearest:
+			for(unsigned int i = 0; i < faces.size(); ++i)
+				new_faces[i] = faces[helpers[i].i];
+			break;
+		case StdSubMeshInstance::FO_NearestToFarthest:
+			for(unsigned int i = 0; i < faces.size(); ++i)
+				new_faces[i] = faces[helpers[faces.size() - i - 1].i];
+			break;
+		default:
+			assert(false);
+			break;
+		}
+
+		faces.swap(new_faces);
+	}
+
 	// Serialize a ValueProvider with StdCompiler
 	struct ValueProviderAdapt
 	{
@@ -153,11 +182,27 @@ namespace
 
 	ValueProviderAdapt mkValueProviderAdapt(StdMeshInstance::ValueProvider** ValueProvider) { return ValueProviderAdapt(ValueProvider); }
 
-	// Serialize a bone index by name with StdCompiler
-	struct TransformAdapt
+	void CompileFloat(StdCompiler* pComp, float& f)
+	{
+		// TODO: Teach StdCompiler how to handle float
+		if(pComp->isCompiler())
+		{
+			C4Real r;
+			pComp->Value(r);
+			f = fixtof(r);
+		}
+		else
+		{
+			C4Real r = ftofix(f);
+			pComp->Value(r);
+		}
+	}
+
+	// Serialize a transformation matrix by name with StdCompiler
+	struct MatrixAdapt
 	{
 		StdMeshMatrix& Matrix;
-		TransformAdapt(StdMeshMatrix& matrix): Matrix(matrix) {}
+		MatrixAdapt(StdMeshMatrix& matrix): Matrix(matrix) {}
 
 		void CompileFunc(StdCompiler* pComp)
 		{
@@ -167,30 +212,42 @@ namespace
 				for(unsigned int j = 0; j < 4; ++j)
 				{
 					if(i != 0 || j != 0) pComp->Separator();
-					// TODO: Teach StdCompiler how to handle float
-//					pComp->Value(Matrix(i, j));
-					
-					if(pComp->isCompiler())
-					{
-						C4Real f;
-						pComp->Value(f);
-						Matrix(i,j) = fixtof(f);
-					}
-					else
-					{
-						C4Real f = ftofix(Matrix(i,j));
-						pComp->Value(f);
-					}
+					CompileFloat(pComp, Matrix(i, j));
 				}
 			}
 
 			pComp->Separator(StdCompiler::SEP_END);
 		}
 
-		ALLOW_TEMP_TO_REF(TransformAdapt)
+		ALLOW_TEMP_TO_REF(MatrixAdapt)
 	};
-	
-	TransformAdapt mkTransformAdapt(StdMeshMatrix& Matrix) { return TransformAdapt(Matrix); }
+
+	struct TransformAdapt
+	{
+		StdMeshTransformation& Trans;
+		TransformAdapt(StdMeshTransformation& trans): Trans(trans) {}
+
+		void CompileFunc(StdCompiler* pComp)
+		{
+			pComp->Separator(StdCompiler::SEP_START);
+			CompileFloat(pComp, Trans.translate.x);
+			CompileFloat(pComp, Trans.translate.y);
+			CompileFloat(pComp, Trans.translate.z);
+			CompileFloat(pComp, Trans.rotate.w);
+			CompileFloat(pComp, Trans.rotate.x);
+			CompileFloat(pComp, Trans.rotate.y);
+			CompileFloat(pComp, Trans.rotate.z);
+			CompileFloat(pComp, Trans.scale.x);
+			CompileFloat(pComp, Trans.scale.y);
+			CompileFloat(pComp, Trans.scale.z);
+			pComp->Separator(StdCompiler::SEP_END);
+		}
+
+		ALLOW_TEMP_TO_REF(TransformAdapt);
+	};
+
+	MatrixAdapt mkMatrixAdapt(StdMeshMatrix& Matrix) { return MatrixAdapt(Matrix); }
+	TransformAdapt mkTransformAdapt(StdMeshTransformation& Trans) { return TransformAdapt(Trans); }
 
 	// Reset all animation list entries corresponding to node or its children
 	void ClearAnimationListRecursively(std::vector<StdMeshInstance::AnimationNode*>& list, StdMeshInstance::AnimationNode* node)
@@ -231,13 +288,6 @@ namespace
 
 		return true;
 	}
-
-	StdMeshInstanceFaceOrderingCmpPred* g_pred = NULL;
-}
-
-static int StdMeshFaceCmp(const StdMeshFace& face1, const StdMeshFace& face2)
-{
-	return g_pred->compare(face1, face2);
 }
 
 StdMeshTransformation StdMeshTrack::GetTransformAt(float time) const
@@ -447,13 +497,16 @@ void StdSubMeshInstance::LoadFacesForCompletion(StdMeshInstance& instance, const
 		// however we can simply give an appropriate transformation matrix to the face ordering.
 		// At this point, all vertices are in the OGRE coordinate frame, and Z in OGRE equals
 		// Y in Clonk, so we are fine without additional transformation.
-		StdMeshInstanceFaceOrderingCmpPred pred(instance, *this, FO_FarthestToNearest, StdMeshMatrix::Identity());
-		g_pred = &pred;
-		StdMesh_tim_sort(&Faces[0], Faces.size());
-		g_pred = NULL;
+		const StdMeshVertex* vertices;
+		if(GetNumVertices() > 0)
+			vertices = &GetVertices()[0];
+		else
+			vertices = &instance.GetSharedVertices()[0];
+		SortFacesArray(vertices, Faces, FO_FarthestToNearest, StdMeshMatrix::Identity());
 
 		// Third: Only use the first few ones
-		Faces.resize(static_cast<unsigned int>(completion * submesh.GetNumFaces() + 0.5));
+		assert(submesh.GetNumFaces() >= 1);
+		Faces.resize(BoundBy<unsigned int>(static_cast<unsigned int>(completion * submesh.GetNumFaces() + 0.5), 1, submesh.GetNumFaces()));
 	}
 }
 
@@ -481,8 +534,6 @@ void StdSubMeshInstance::SetMaterial(const StdMeshMaterial& material)
 		}
 	}
 
-	// TODO: Reorder this submesh so that opaque submeshes are drawn
-	// before non-opaque ones.
 	// TODO: Reset face ordering
 }
 
@@ -511,6 +562,33 @@ void StdSubMeshInstance::SetFaceOrderingForClrModulation(const StdSubMesh& subme
 		SetFaceOrdering(submesh, FO_Fixed);
 }
 
+void StdSubMeshInstance::CompileFunc(StdCompiler* pComp)
+{
+	// TODO: We should also serialize the texture animation positions
+	if(pComp->isCompiler())
+	{
+		StdCopyStrBuf material_name;
+		pComp->Value(mkNamingAdapt(material_name, "Material"));
+		if(material_name != Material->Name)
+		{
+			const StdMeshMaterial* material = ::MeshMaterialManager.GetMaterial(material_name.getData());
+			if(!material)
+			{
+				StdStrBuf buf;
+				buf.Format("There is no such material with name \"%s\"", material_name.getData());
+				pComp->excCorrupt(buf.getData());
+			}
+
+			SetMaterial(*material);
+		}
+	}
+	else
+	{
+		StdCopyStrBuf material_name = Material->Name;
+		pComp->Value(mkNamingAdapt(material_name, "Material"));
+	}
+}
+
 void StdMeshInstance::SerializableValueProvider::CompileFunc(StdCompiler* pComp)
 {
 	pComp->Value(Value);
@@ -530,6 +608,13 @@ StdMeshInstance::AnimationNode::AnimationNode(const StdMeshAnimation* animation,
 	Leaf.Position = position;
 }
 
+StdMeshInstance::AnimationNode::AnimationNode(const StdMeshBone* bone, const StdMeshTransformation& trans):
+		Type(CustomNode), Parent(NULL)
+{
+	Custom.BoneIndex = bone->Index;
+	Custom.Transformation = new StdMeshTransformation(trans);
+}
+
 StdMeshInstance::AnimationNode::AnimationNode(AnimationNode* child_left, AnimationNode* child_right, ValueProvider* weight):
 		Type(LinearInterpolationNode), Parent(NULL)
 {
@@ -545,6 +630,9 @@ StdMeshInstance::AnimationNode::~AnimationNode()
 	case LeafNode:
 		delete Leaf.Position;
 		break;
+	case CustomNode:
+		delete Custom.Transformation;
+		break;
 	case LinearInterpolationNode:
 		delete LinearInterpolation.ChildLeft;
 		delete LinearInterpolation.ChildRight;
@@ -566,6 +654,12 @@ bool StdMeshInstance::AnimationNode::GetBoneTransform(unsigned int bone, StdMesh
 		if (!track) return false;
 		transformation = track->GetTransformAt(fixtof(Leaf.Position->Value));
 		return true;
+	case CustomNode:
+		if(bone == Custom.BoneIndex)
+			transformation = *Custom.Transformation;
+		else
+			return false;
+		return true;
 	case LinearInterpolationNode:
 		if (!LinearInterpolation.ChildLeft->GetBoneTransform(bone, transformation))
 			return LinearInterpolation.ChildRight->GetBoneTransform(bone, transformation);
@@ -585,6 +679,7 @@ void StdMeshInstance::AnimationNode::CompileFunc(StdCompiler* pComp, const StdMe
 	static const StdEnumEntry<NodeType> NodeTypes[] =
 	{
 		{ "Leaf",                  LeafNode                      },
+		{ "Custom",                CustomNode                    },
 		{ "LinearInterpolation",   LinearInterpolationNode       },
 
 		{ NULL,     static_cast<NodeType>(0)  }
@@ -611,6 +706,22 @@ void StdMeshInstance::AnimationNode::CompileFunc(StdCompiler* pComp, const StdMe
 
 		pComp->Value(mkNamingAdapt(mkValueProviderAdapt(&Leaf.Position), "Position"));
 		break;
+	case CustomNode:
+		if(pComp->isCompiler())
+		{
+			StdCopyStrBuf bone_name;
+			pComp->Value(mkNamingAdapt(toC4CStrBuf(bone_name), "Bone"));
+			const StdMeshBone* bone = Mesh->GetBoneByName(bone_name);
+			if(!bone) pComp->excCorrupt("No such bone: \"%s\"", bone_name.getData());
+			Custom.BoneIndex = bone->Index;
+		}
+		else
+		{
+			pComp->Value(mkNamingAdapt(mkParAdapt(mkDecompileAdapt(Mesh->GetBone(Custom.BoneIndex).Name), StdCompiler::RCT_All), "Bone"));
+		}
+
+		pComp->Value(mkNamingAdapt(mkTransformAdapt(*Custom.Transformation), "Transformation"));
+		break;
 	case LinearInterpolationNode:
 		pComp->Value(mkParAdapt(mkNamingPtrAdapt(LinearInterpolation.ChildLeft, "ChildLeft"), Mesh));
 		pComp->Value(mkParAdapt(mkNamingPtrAdapt(LinearInterpolation.ChildRight, "ChildRight"), Mesh));
@@ -621,7 +732,7 @@ void StdMeshInstance::AnimationNode::CompileFunc(StdCompiler* pComp, const StdMe
 				pComp->excCorrupt("Slot of left child does not match parent slot");
 			if(LinearInterpolation.ChildRight->Slot != Slot)
 				pComp->excCorrupt("Slof of right child does not match parent slot");
-			LinearInterpolation.ChildRight->Parent = this;
+			LinearInterpolation.ChildLeft->Parent = this;
 			LinearInterpolation.ChildRight->Parent = this;
 		}
 		break;
@@ -639,14 +750,38 @@ void StdMeshInstance::AnimationNode::DenumeratePointers()
 	case LeafNode:
 		value_provider = dynamic_cast<SerializableValueProvider*>(Leaf.Position);
 		break;
+	case CustomNode:
+		value_provider = NULL;
+		break;
 	case LinearInterpolationNode:
 		value_provider = dynamic_cast<SerializableValueProvider*>(LinearInterpolation.Weight);
+		// non-recursive, StdMeshInstance::DenumeratePointers walks over all nodes
 		break;
 	}
 
 	if(value_provider) value_provider->DenumeratePointers();
 }
 
+void StdMeshInstance::AnimationNode::ClearPointers(class C4Object* pObj)
+{
+	SerializableValueProvider* value_provider = NULL;
+	switch(Type)
+	{
+	case LeafNode:
+		value_provider = dynamic_cast<SerializableValueProvider*>(Leaf.Position);
+		break;
+	case CustomNode:
+		value_provider = NULL;
+		break;
+	case LinearInterpolationNode:
+		value_provider = dynamic_cast<SerializableValueProvider*>(LinearInterpolation.Weight);
+		// non-recursive, StdMeshInstance::ClearPointers walks over all nodes
+		break;
+	}
+
+	if(value_provider) value_provider->ClearPointers(pObj);
+}
+
 StdMeshInstance::AttachedMesh::AttachedMesh():
 	Number(0), Parent(NULL), Child(NULL), OwnChild(true), ChildDenumerator(NULL), ParentBone(0), ChildBone(0), FinalTransformDirty(false)
 {
@@ -711,8 +846,8 @@ void StdMeshInstance::AttachedMesh::CompileFunc(StdCompiler* pComp, DenumeratorF
 	pComp->Value(mkNamingAdapt(Number, "Number"));
 	pComp->Value(mkNamingAdapt(ParentBone, "ParentBone")); // TODO: Save as string
 	pComp->Value(mkNamingAdapt(ChildBone, "ChildBone")); // TODO: Save as string (note we can only resolve this in DenumeratePointers then!)
-	pComp->Value(mkNamingAdapt(mkTransformAdapt(AttachTrans), "AttachTransformation"));
-	
+	pComp->Value(mkNamingAdapt(mkMatrixAdapt(AttachTrans), "AttachTransformation"));
+
 	uint8_t dwSyncFlags = static_cast<uint8_t>(Flags);
 	pComp->Value(mkNamingAdapt(mkBitfieldAdapt(dwSyncFlags, AM_Entries), "Flags", 0u));
 	if(pComp->isCompiler()) Flags = dwSyncFlags;
@@ -725,6 +860,11 @@ void StdMeshInstance::AttachedMesh::DenumeratePointers()
 	ChildDenumerator->DenumeratePointers(this);
 }
 
+bool StdMeshInstance::AttachedMesh::ClearPointers(class C4Object* pObj)
+{
+	return ChildDenumerator->ClearPointers(pObj);
+}
+
 StdMeshInstance::StdMeshInstance(const StdMesh& mesh, float completion):
 		Mesh(&mesh), SharedVertices(mesh.GetSharedVertices().size()), Completion(completion),
 		BoneTransforms(Mesh->GetNumBones(), StdMeshMatrix::Identity()),
@@ -741,6 +881,9 @@ StdMeshInstance::StdMeshInstance(const StdMesh& mesh, float completion):
 		const StdSubMesh& submesh = Mesh->GetSubMesh(i);
 		SubMeshInstances[i] = new StdSubMeshInstance(*this, submesh, completion);
 	}
+
+	// copy, order is fine at the moment since only default materials are used.
+	SubMeshInstancesOrdered = SubMeshInstances;
 }
 
 StdMeshInstance::~StdMeshInstance()
@@ -806,70 +949,16 @@ StdMeshInstance::AnimationNode* StdMeshInstance::PlayAnimation(const StdStrBuf&
 
 StdMeshInstance::AnimationNode* StdMeshInstance::PlayAnimation(const StdMeshAnimation& animation, int slot, AnimationNode* sibling, ValueProvider* position, ValueProvider* weight)
 {
-	// Default
-	if (!sibling) sibling = GetRootAnimationForSlot(slot);
-	assert(!sibling || sibling->Slot == slot);
-
-	// Find two subsequent numbers in case we need to create two nodes, so
-	// script can deduce the second node.
-	unsigned int Number1, Number2;
-	for (Number1 = 0; Number1 < AnimationNodes.size(); ++Number1)
-		if (AnimationNodes[Number1] == NULL && (!sibling || Number1+1 == AnimationNodes.size() || AnimationNodes[Number1+1] == NULL))
-			break;
-	/*  for(Number2 = Number1+1; Number2 < AnimationNodes.size(); ++Number2)
-	    if(AnimationNodes[Number2] == NULL)
-	      break;*/
-	Number2 = Number1 + 1;
-
 	position->Value = BoundBy(position->Value, Fix0, ftofix(animation.Length));
-	weight->Value = BoundBy(weight->Value, Fix0, itofix(1));
-
-	if (Number1 == AnimationNodes.size()) AnimationNodes.push_back( (StdMeshInstance::AnimationNode*) NULL);
-	if (sibling && Number2 == AnimationNodes.size()) AnimationNodes.push_back( (StdMeshInstance::AnimationNode*) NULL);
-
 	AnimationNode* child = new AnimationNode(&animation, position);
-	AnimationNodes[Number1] = child;
-	child->Number = Number1;
-	child->Slot = slot;
+	InsertAnimationNode(child, slot, sibling, weight);
+	return child;
+}
 
-	if (sibling)
-	{
-		AnimationNode* parent = new AnimationNode(child, sibling, weight);
-		AnimationNodes[Number2] = parent;
-		parent->Number = Number2;
-		parent->Slot = slot;
-
-		child->Parent = parent;
-		parent->Parent = sibling->Parent;
-		parent->LinearInterpolation.ChildLeft = sibling;
-		parent->LinearInterpolation.ChildRight = child;
-		if (sibling->Parent)
-		{
-			if (sibling->Parent->LinearInterpolation.ChildLeft == sibling)
-				sibling->Parent->LinearInterpolation.ChildLeft = parent;
-			else
-				sibling->Parent->LinearInterpolation.ChildRight = parent;
-		}
-		else
-		{
-			// set new parent
-			AnimationNodeList::iterator iter = GetStackIterForSlot(slot, false);
-			// slot must not be empty, since sibling uses same slot
-			assert(iter != AnimationStack.end() && *iter != NULL);
-			*iter = parent;
-		}
-
-		sibling->Parent = parent;
-	}
-	else
-	{
-		delete weight;
-		AnimationNodeList::iterator iter = GetStackIterForSlot(slot, true);
-		assert(!*iter); // we have a sibling if slot is not empty
-		*iter = child;
-	}
-
-	BoneTransformsDirty = true;
+StdMeshInstance::AnimationNode* StdMeshInstance::PlayAnimation(const StdMeshBone* bone, const StdMeshTransformation& trans, int slot, AnimationNode* sibling, ValueProvider* weight)
+{
+	AnimationNode* child = new AnimationNode(bone, trans);
+	InsertAnimationNode(child, slot, sibling, weight);
 	return child;
 }
 
@@ -954,6 +1043,13 @@ void StdMeshInstance::SetAnimationPosition(AnimationNode* node, ValueProvider* p
 	BoneTransformsDirty = true;
 }
 
+void StdMeshInstance::SetAnimationBoneTransform(AnimationNode* node, const StdMeshTransformation& trans)
+{
+	assert(node->GetType() == AnimationNode::CustomNode);
+	*node->Custom.Transformation = trans;
+	BoneTransformsDirty = true;
+}
+
 void StdMeshInstance::SetAnimationWeight(AnimationNode* node, ValueProvider* weight)
 {
 	assert(node->GetType() == AnimationNode::LinearInterpolationNode);
@@ -1017,7 +1113,7 @@ StdMeshInstance::AttachedMesh* StdMeshInstance::AttachMesh(const StdMesh& mesh,
 
 StdMeshInstance::AttachedMesh* StdMeshInstance::AttachMesh(StdMeshInstance& instance, AttachedMesh::Denumerator* denumerator, const StdStrBuf& parent_bone, const StdStrBuf& child_bone, const StdMeshMatrix& transformation, uint32_t flags, bool own_child)
 {
-	std::auto_ptr<AttachedMesh::Denumerator> auto_denumerator(denumerator);
+	std::unique_ptr<AttachedMesh::Denumerator> auto_denumerator(denumerator);
 
 	// We don't allow an instance to be attached to multiple parent instances for now
 	if (instance.AttachParent) return NULL;
@@ -1078,6 +1174,13 @@ StdMeshInstance::AttachedMesh* StdMeshInstance::GetAttachedMeshByNumber(unsigned
 	return NULL;
 }
 
+void StdMeshInstance::SetMaterial(size_t i, const StdMeshMaterial& material)
+{
+	assert(i < SubMeshInstances.size());
+	SubMeshInstances[i]->SetMaterial(material);
+	std::stable_sort(SubMeshInstancesOrdered.begin(), SubMeshInstancesOrdered.end(), StdMeshSubMeshInstanceVisibilityCmpPred());
+}
+
 bool StdMeshInstance::UpdateBoneTransforms()
 {
 	bool was_dirty = BoneTransformsDirty;
@@ -1125,7 +1228,7 @@ bool StdMeshInstance::UpdateBoneTransforms()
 
 		// Compute transformation for each vertex. We could later think about
 		// doing this on the GPU using a vertex shader. This would then probably
-		// need to go to CStdGL::PerformMesh and CStdD3D::PerformMesh.
+		// need to go to CStdGL::PerformMesh.
 		// But first, we need to move vertex data to the GPU.
 		if(!Mesh->GetSharedVertices().empty())
 			ApplyBoneTransformToVertices(Mesh->GetSharedVertices(), SharedVertices);
@@ -1178,26 +1281,16 @@ void StdMeshInstance::ReorderFaces(StdMeshMatrix* global_trans)
 	for (unsigned int i = 0; i < SubMeshInstances.size(); ++i)
 	{
 		StdSubMeshInstance& inst = *SubMeshInstances[i];
-		if(inst.CurrentFaceOrdering != StdSubMeshInstance::FO_Fixed)
+		assert((inst.Faces.size() > 0) && "StdMeshInstance sub-mesh instance has zero faces");
+
+		if(inst.Faces.size() > 0 && inst.CurrentFaceOrdering != StdSubMeshInstance::FO_Fixed)
 		{
-			StdMeshInstanceFaceOrderingCmpPred pred(*this, inst, inst.CurrentFaceOrdering, global_trans ? *global_trans : StdMeshMatrix::Identity());
-
-			// The usage of timsort instead of std::sort at this point is twofold.
-			// First, it's faster in our case where the array is already sorted in
-			// many cases (remember this is called at least once a frame).
-			// And it's not just a bit faster either but a lot. I have measured
-			// a factor of 7 on my system.
-			// Second, in our Windows autobuilds there is a crash within std::sort
-			// which is very hard to debug because it's hardly reproducible with
-			// anything other than the autobuilds (I tried hard). If the crash goes
-			// away with timsort then great, if not then maybe it's easier to debug
-			// since the code is in our tree.
-
-			//std::sort(inst.Faces.begin(), inst.Faces.end(), pred);
-
-			g_pred = &pred;
-			StdMesh_tim_sort(&inst.Faces[0], inst.Faces.size());
-			g_pred = NULL;
+			const StdMeshVertex* vertices;
+			if(inst.GetNumVertices() > 0)
+				vertices = &inst.GetVertices()[0];
+			else
+				vertices = &GetSharedVertices()[0];
+			SortFacesArray(vertices, inst.Faces, inst.CurrentFaceOrdering, global_trans ? *global_trans : StdMeshMatrix::Identity());
 		}
 	}
 
@@ -1218,6 +1311,14 @@ void StdMeshInstance::CompileFunc(StdCompiler* pComp, AttachedMesh::DenumeratorF
 		pComp->Value(mkNamingAdapt(valid, "Valid"));
 		if(!valid) pComp->excCorrupt("Mesh instance is invalid");
 
+		int32_t iSubMeshCnt;
+		pComp->Value(mkNamingCountAdapt(iSubMeshCnt, "SubMesh"));
+		if(static_cast<uint32_t>(iSubMeshCnt) != SubMeshInstances.size())
+			pComp->excCorrupt("Invalid number of submeshes");
+		for(int32_t i = 0; i < iSubMeshCnt; ++i)
+			pComp->Value(mkNamingAdapt(*SubMeshInstances[i], "SubMesh"));
+		std::stable_sort(SubMeshInstancesOrdered.begin(), SubMeshInstancesOrdered.end(), StdMeshSubMeshInstanceVisibilityCmpPred());
+
 		int32_t iAnimCnt = AnimationStack.size();
 		pComp->Value(mkNamingCountAdapt(iAnimCnt, "AnimationNode"));
 
@@ -1270,6 +1371,11 @@ void StdMeshInstance::CompileFunc(StdCompiler* pComp, AttachedMesh::DenumeratorF
 		bool valid = true;
 		pComp->Value(mkNamingAdapt(valid, "Valid"));
 
+		int32_t iSubMeshCnt = SubMeshInstances.size();
+		pComp->Value(mkNamingCountAdapt(iSubMeshCnt, "SubMesh"));
+		for(int32_t i = 0; i < iSubMeshCnt; ++i)
+			pComp->Value(mkNamingAdapt(*SubMeshInstances[i], "SubMesh"));
+
 		int32_t iAnimCnt = AnimationStack.size();
 		pComp->Value(mkNamingCountAdapt(iAnimCnt, "AnimationNode"));
 
@@ -1294,6 +1400,21 @@ void StdMeshInstance::DenumeratePointers()
 		AttachChildren[i]->DenumeratePointers();
 }
 
+void StdMeshInstance::ClearPointers(class C4Object* pObj)
+{
+	for(unsigned int i = 0; i < AnimationNodes.size(); ++i)
+		if(AnimationNodes[i])
+			AnimationNodes[i]->ClearPointers(pObj);
+
+	std::vector<unsigned int> Removal;
+	for(unsigned int i = 0; i < AttachChildren.size(); ++i)
+		if(!AttachChildren[i]->ClearPointers(pObj))
+			Removal.push_back(AttachChildren[i]->Number);
+
+	for(unsigned int i = 0; i < Removal.size(); ++i)
+		DetachMesh(Removal[i]);
+}
+
 StdMeshInstance::AnimationNodeList::iterator StdMeshInstance::GetStackIterForSlot(int slot, bool create)
 {
 	// TODO: bsearch
@@ -1318,6 +1439,72 @@ StdMeshInstance::AnimationNodeList::iterator StdMeshInstance::GetStackIterForSlo
 		return AnimationStack.insert(AnimationStack.end(), NULL);
 }
 
+void StdMeshInstance::InsertAnimationNode(AnimationNode* node, int slot, AnimationNode* sibling, ValueProvider* weight)
+{
+	// Default
+	if (!sibling) sibling = GetRootAnimationForSlot(slot);
+	assert(!sibling || sibling->Slot == slot);
+
+	// Find two subsequent numbers in case we need to create two nodes, so
+	// script can deduce the second node.
+	unsigned int Number1, Number2;
+	for (Number1 = 0; Number1 < AnimationNodes.size(); ++Number1)
+		if (AnimationNodes[Number1] == NULL && (!sibling || Number1+1 == AnimationNodes.size() || AnimationNodes[Number1+1] == NULL))
+			break;
+	/*  for(Number2 = Number1+1; Number2 < AnimationNodes.size(); ++Number2)
+	    if(AnimationNodes[Number2] == NULL)
+	      break;*/
+	Number2 = Number1 + 1;
+
+	weight->Value = BoundBy(weight->Value, Fix0, itofix(1));
+
+	if (Number1 == AnimationNodes.size()) AnimationNodes.push_back( (StdMeshInstance::AnimationNode*) NULL);
+	if (sibling && Number2 == AnimationNodes.size()) AnimationNodes.push_back( (StdMeshInstance::AnimationNode*) NULL);
+
+	AnimationNodes[Number1] = node;
+	node->Number = Number1;
+	node->Slot = slot;
+
+	if (sibling)
+	{
+		AnimationNode* parent = new AnimationNode(node, sibling, weight);
+		AnimationNodes[Number2] = parent;
+		parent->Number = Number2;
+		parent->Slot = slot;
+
+		node->Parent = parent;
+		parent->Parent = sibling->Parent;
+		parent->LinearInterpolation.ChildLeft = sibling;
+		parent->LinearInterpolation.ChildRight = node;
+		if (sibling->Parent)
+		{
+			if (sibling->Parent->LinearInterpolation.ChildLeft == sibling)
+				sibling->Parent->LinearInterpolation.ChildLeft = parent;
+			else
+				sibling->Parent->LinearInterpolation.ChildRight = parent;
+		}
+		else
+		{
+			// set new parent
+			AnimationNodeList::iterator iter = GetStackIterForSlot(slot, false);
+			// slot must not be empty, since sibling uses same slot
+			assert(iter != AnimationStack.end() && *iter != NULL);
+			*iter = parent;
+		}
+
+		sibling->Parent = parent;
+	}
+	else
+	{
+		delete weight;
+		AnimationNodeList::iterator iter = GetStackIterForSlot(slot, true);
+		assert(!*iter); // we have a sibling if slot is not empty
+		*iter = node;
+	}
+
+	BoneTransformsDirty = true;
+}
+
 bool StdMeshInstance::ExecuteAnimationNode(AnimationNode* node)
 {
 	ValueProvider* provider = NULL;
@@ -1331,6 +1518,9 @@ bool StdMeshInstance::ExecuteAnimationNode(AnimationNode* node)
 		min = Fix0;
 		max = ftofix(node->GetAnimation()->Length);
 		break;
+	case AnimationNode::CustomNode:
+		// No execution necessary
+		return true;
 	case AnimationNode::LinearInterpolationNode:
 		provider = node->GetWeightProvider();
 		min = Fix0;
@@ -1340,6 +1530,8 @@ bool StdMeshInstance::ExecuteAnimationNode(AnimationNode* node)
 		assert(false);
 		break;
 	}
+
+	assert(provider);
 	const C4Real old_value = provider->Value;
 
 	if (!provider->Execute())
