@@ -1,26 +1,18 @@
 /*
  * OpenClonk, http://www.openclonk.org
  *
- * Copyright (c) 1998-2000, 2003-2005, 2008  Matthes Bender
- * Copyright (c) 2001  Michael Käser
- * Copyright (c) 2001-2008  Sven Eberhardt
- * Copyright (c) 2002, 2004, 2006  Peter Wortmann
- * Copyright (c) 2004-2006, 2008-2009, 2011  Günther Brammer
- * Copyright (c) 2009  Nicolas Hake
- * Copyright (c) 2010  Benjamin Herr
- * Copyright (c) 2010  Armin Burgmeier
- * Copyright (c) 2001-2009, RedWolf Design GmbH, http://www.clonk.de
+ * Copyright (c) 1998-2000, Matthes Bender
+ * Copyright (c) 2001-2009, RedWolf Design GmbH, http://www.clonk.de/
+ * Copyright (c) 2009-2013, The OpenClonk Team and contributors
  *
- * Portions might be copyrighted by other authors who have contributed
- * to OpenClonk.
+ * Distributed under the terms of the ISC license; see accompanying file
+ * "COPYING" for details.
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- * See isc_license.txt for full license and disclaimer.
+ * "Clonk" is a registered trademark of Matthes Bender, used with permission.
+ * See accompanying file "TRADEMARK" for details.
  *
- * "Clonk" is a registered trademark of Matthes Bender.
- * See clonk_trademark_license.txt for full license.
+ * To redistribute this file separately, substitute the full license texts
+ * for the above references.
  */
 
 /* The command stack controls an object's complex and independent behavior */
@@ -139,11 +131,21 @@ int32_t CommandByName(const char *szCommand)
 	return C4CMD_None;
 }
 
+bool FreeMoveTo(C4Object *cObj)
+{
+	// Floating: we accept any move-to target
+	if (cObj->GetProcedure()==DFA_FLOAT) return true;
+	// Can fly: we accept any move-to target
+	//if (cObj->GetPhysical()->CanFly) return true; - needs to be adjusted once we have dragons
+	// Assume we're walking: move-to targets are adjusted
+	return false;
+}
+
 void AdjustMoveToTarget(int32_t &rX, int32_t &rY, bool fFreeMove, int32_t iShapeHgt)
 {
 	// Above solid (always)
-	int32_t iY;
-	for (iY=rY; (iY>=0) && GBackSolid(rX,iY); iY--) {}
+	int32_t iY=Min(rY, GBackHgt);
+	while ((iY>=0) && GBackSolid(rX,iY)) iY--;
 	if (iY>=0) rY=iY;
 	// No-free-move adjustments (i.e. if walking)
 	if (!fFreeMove)
@@ -422,7 +424,7 @@ void C4Command::MoveTo()
 		// difference to momentum
 		dx -= cObj->xdir; dy -= cObj->ydir;
 		// steer
-		if (Abs(dx)+Abs(dy) < C4REAL100(20)) cObj->Action.ComDir = COMD_None;
+		if (Abs(dx)+Abs(dy) < C4REAL100(20)) cObj->Action.ComDir = COMD_Stop;
 		else if (Abs(dy) * 3 <  dx) cObj->Action.ComDir = COMD_Right;
 		else if (Abs(dy) * 3 < -dx) cObj->Action.ComDir = COMD_Left;
 		else if (Abs(dx) * 3 <  dy) cObj->Action.ComDir = COMD_Down;
@@ -948,7 +950,7 @@ void C4Command::Get()
 
 	// Get target specified by container and type
 	if (!Target && Target2 && Data)
-		if (!(Target = Target2->Contents.Find(Data.getC4ID())))
+		if (!(Target = Target2->Contents.Find(Data.getDef())))
 			{ Finish(); return; }
 
 	// No target: failure
@@ -1124,7 +1126,7 @@ void C4Command::Activate()
 			C4Object *pObj; C4ObjectLink *cLnk;
 			if (!Target)
 				for (cLnk=Target2->Contents.First; cLnk && (pObj=cLnk->Obj); cLnk=cLnk->Next)
-					if (pObj->Status && (pObj->Def->id==Data.getC4ID()))
+					if (pObj->Status && (pObj->Def==Data.getDef()))
 						if (!pObj->Command || (pObj->Command->Command!=C4CMD_Exit))
 							{ Target=pObj; break; }
 			// No target
@@ -1163,7 +1165,7 @@ void C4Command::Put() // Notice: Put command is currently using Ty as an interna
 
 	// Thing to put specified by type
 	if (!Target2 && Data)
-		if (!(Target2 = cObj->Contents.Find(Data.getC4ID())))
+		if (!(Target2 = cObj->Contents.Find(Data.getDef())))
 			{ Finish(); return; }
 
 	// No thing to put specified
@@ -1391,7 +1393,7 @@ bool C4Command::InitEvaluation()
 		if (Target) { Tx+=Target->GetX(); Ty+=Target->GetY(); Target=NULL; }
 		// Adjust coordinates
 		int32_t iTx = Tx._getInt();
-		if (~Data.getInt() & C4CMD_MoveTo_NoPosAdjust) AdjustMoveToTarget(iTx,Ty,true,cObj->Shape.Hgt);
+		if (~Data.getInt() & C4CMD_MoveTo_NoPosAdjust) AdjustMoveToTarget(iTx,Ty,FreeMoveTo(cObj),cObj->Shape.Hgt);
 		Tx.SetInt(iTx);
 		return true;
 	}
@@ -1400,7 +1402,7 @@ bool C4Command::InitEvaluation()
 	{
 		// Adjust coordinates
 		int32_t iTx = Tx._getInt();
-		AdjustMoveToTarget(iTx,Ty,true,cObj->Shape.Hgt);
+		AdjustMoveToTarget(iTx,Ty,FreeMoveTo(cObj),cObj->Shape.Hgt);
 		Tx.SetInt(iTx);
 		return true;
 	}
@@ -1412,7 +1414,7 @@ bool C4Command::InitEvaluation()
 		// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 	case C4CMD_Wait:
 		// Update interval by Data
-		if (!Data) UpdateInterval=Data.getInt();
+		if (!!Data) UpdateInterval=Data.getInt();
 		// Else update interval by Tx
 		else if (Tx._getInt()) UpdateInterval=Tx._getInt();
 		return true;
@@ -1523,17 +1525,21 @@ bool C4Command::JumpControl() // Called by DFA_WALK
 		}
 
 	// Low side contact jump
+	// Only jump before almost running off a cliff
 	int32_t iLowSideRange=5;
-	if (cObj->t_contact & CNAT_Right)
-		if (Inside(iAngle-JumpLowAngle,-iLowSideRange*JumpAngleRange,+iLowSideRange*JumpAngleRange))
-		{
-			cObj->AddCommand(C4CMD_Jump,NULL,Tx,Ty); return true;
-		}
-	if (cObj->t_contact & CNAT_Left)
-		if (Inside(iAngle+JumpLowAngle,-iLowSideRange*JumpAngleRange,+iLowSideRange*JumpAngleRange))
-		{
-			cObj->AddCommand(C4CMD_Jump,NULL,Tx,Ty); return true;
-		}
+	if (!(GBackDensity(cx,cy+cObj->Shape.Hgt/2) >= cObj->Shape.ContactDensity))
+	{
+		if (cObj->t_contact & CNAT_Right)
+			if (Inside(iAngle-JumpLowAngle,-iLowSideRange*JumpAngleRange,+iLowSideRange*JumpAngleRange))
+			{
+				cObj->AddCommand(C4CMD_Jump,NULL,Tx,Ty); return true;
+			}
+		if (cObj->t_contact & CNAT_Left)
+			if (Inside(iAngle+JumpLowAngle,-iLowSideRange*JumpAngleRange,+iLowSideRange*JumpAngleRange))
+			{
+				cObj->AddCommand(C4CMD_Jump,NULL,Tx,Ty); return true;
+			}
+	}
 
 	// No jump control
 	return false;
@@ -1631,7 +1637,7 @@ void C4Command::Acquire()
 	if (!Data) { Finish(); return; }
 
 	// Target material in inventory: done
-	if (cObj->Contents.Find(Data.getC4ID()))
+	if (cObj->Contents.Find(Data.getDef()))
 		{ Finish(true); return; }
 
 	// script overload
@@ -1648,7 +1654,7 @@ void C4Command::Acquire()
 	// Find available material
 	C4Object *pMaterial=NULL;
 	// Next closest
-	while ((pMaterial = Game.FindObject(Data.getC4ID(),cObj->GetX(),cObj->GetY(),-1,-1,OCF_Available,pMaterial)))
+	while ((pMaterial = Game.FindObject(Data.getDef(),cObj->GetX(),cObj->GetY(),-1,-1,OCF_Available,pMaterial)))
 		// Object is not in container to be ignored
 		if (!Target2 || pMaterial->Contained!=Target2)
 			// Object is near enough
@@ -1741,7 +1747,7 @@ void C4Command::Fail(const char *szFailMessage)
 			if (szFailMessage) break;
 			// Fail message with name of target type
 			SCopy(LoadResStr(CommandNameID(Command)), szCommandName);
-			C4Def *pDef; pDef = ::Definitions.ID2Def(Data.getC4ID());
+			C4Def *pDef; pDef = Data.getDef();
 			SCopy(pDef ? pDef->GetName() : LoadResStr("IDS_OBJ_UNKNOWN"), szObjectName);
 			str.Format(LoadResStr("IDS_CON_FAILUREOF"), szCommandName, szObjectName);
 			break;
